# encoding: utf-8
import os
import sys
import getpass
import urllib
from ConfigParser import ConfigParser
#from utils import parse_ocapi
from config import EDO_CONFIG_PATH, OC_API, SECTION, ACCOUNT, INSTANCE, \
    USERNAME, CLIENT_ID, CLIENT_SECRET

class EdoEnvironment:

    def __init__(self, path=EDO_CONFIG_PATH):
        self.path = path
        self.edo_configparse = ConfigParser()
        cf = self.edo_configparse
        if not os.path.exists(path):
            #  初始化配置文件
            section= read_init_section(SECTION)
            oc_api, account, instance, username, password = read_detail_inputs()
            cf.add_section('edopkg')
            cf.set('edopkg', 'server', section)
            self.save_config(section, oc_api, account, instance, username, password)
        cf.read(path)

    def server(self, section):
        if section in self.edo_configparse.sections():
            # 重新设置服务器配置
            self.edo_configparse.set('edopkg', 'server', section)
            self.edo_configparse.write(open(self.path, 'w'))
            print u'修改成功'
        elif section:
            print u'配置名称 %s 不存在' % section

        #  显示当前服务器配置
        self.print_server_config()

    def config(self):
        self.print_server_config()
        action = read_action_input()
        cf = self.edo_configparse
        if action == 'modify':
            section = read_section_input(action, cf.sections(),
                                         cf.get('edopkg', 'server'))
            edo_config = dict(cf.items(section))
            oc_api, account, instance, username, password = read_detail_inputs(
                oc_api=edo_config['oc_api'],
                account=edo_config['account'],
                instance=edo_config['instance'],
                username=edo_config['username']
            )
        else:
            oc_api, account, instance, username, password = read_detail_inputs()
            section = read_section_input(action, cf.sections(), SECTION)

        # 保存配置
        self.save_config(section, oc_api, account, instance, username, password)

    def print_server_config(self):
        cf = self.edo_configparse
        active_section = cf.get('edopkg', 'server')
        print u'当前的默认服务器配置为 %s' % active_section
        info_template = u'%s, 账号 %s, 站点 %s '
        for section in cf.sections():
            if section == 'edopkg':
                continue
            oc_api = cf.get(section, 'oc_api')
            account = cf.get(section, 'account')
            instance = cf.get(section, 'instance')
            if section == active_section:
                head = u'  * %s:' % section
            else:
                head = u'    %s:' % section
            print ''.join([head.ljust(20), info_template % (oc_api, account, instance)])

    def save_config(self, section, oc_api, account, instance, username, password):
        cf = self.edo_configparse
        if section not in cf.sections():
            cf.add_section(section)
        cf.set(section, 'client_id', CLIENT_ID)
        cf.set(section, 'client_secret', CLIENT_SECRET)
        cf.set(section, 'oc_api', oc_api)
        cf.set(section, 'account', account)
        cf.set(section, 'instance', instance)
        cf.set(section, 'username', username)
        cf.set(section, 'password', password)
        cf.write(open(self.path, 'w'))
        print u' 配置文件已保存到：%s ' % self.path

    def load_config(self, section=''):
        if not section:
            section = self.edo_configparse.get('edopkg', 'server')
        cf = dict(self.edo_configparse.items(section))
        return cf['oc_api'], cf['account'], cf['instance'], cf['username'], \
            cf['password'], cf['client_id'], cf['client_secret']

def read_action_input():
    print u'''您可以选择：
    1.修改已存在的配置
    2.新建一个配置
    '''
    sys.stdout.write(u'您的选择[1/2](或输入其他命令退出)：')
    num = raw_input()
    if num not in ['1', '2']:
        sys.exit(0)
    return 'modify' if num == '1' else 'add'

def read_section_input(action, sections, default=''):
    while True:
        section = get_input(u'请输入配置名', default=default)
        # 检查用户输入的配置名
        if action == 'modify' and section not in sections:
            print u'不存在此配置，请确认后重新输入'
        elif action == 'add' and section in sections:
            print u'配置已存在，请确认后重新输入'
        elif not section:
            print u'配置名不能为空，请重新输入'
        else:
            break
    return section

def read_init_section(section):
    print u'''
    -----------------------------
    需要输入配置信息以进行初始化
    -----------------------------'''
    section = get_input(u'请输入配置名', default=section)
    return section

def read_detail_inputs(oc_api=OC_API, account=ACCOUNT,instance=INSTANCE,
                       username=USERNAME):
    print u'''
 -----------------------------
 请输入具体的配置数据
 -----------------------------
 '''
    while True:
        oc_api = parse_ocapi(get_input(u'oc_api(oc服务地址)', default=oc_api))
        account = get_input(u'account(账号名称)', default=account)
        instance = get_input(u'instance(站点实例)', default=instance)
        username = get_input(u'username(用户名)', default=username)

        print u'''
 -----------------------------
 您的输入：
 -----------------------------'''
        print ' oc_api: %s' % oc_api
        print ' account: %s' % account
        print ' instance: %s' % instance
        print ' username: %s' % username
        print u'''
 -----------------------------
 您确认要输入的是以上的数据么？[y/n]（确认：y | 重新输入：n）：'''
        if raw_input().lower() in ['y', '']:
            break
        else:
            print u'请重新输入数据:'

    while True:
        password = getpass.getpass('password: ')
        confirm_pwd = getpass.getpass('confirm password: ')
        if password == confirm_pwd:
            break
        else:
            print u'两次输入的密码不一致,  请重新输入'
    return oc_api, account, instance, username, password

def get_input(msg, default =''):
    msg = ' %s[%s]: ' % (msg, default)
    sys.stdout.write(msg)
    input_info = raw_input()
    return input_info if input_info else default

def parse_ocapi(oc_api):
    scheme, rest = urllib.splittype(oc_api)
    host = urllib.splithost(rest)[0]
    h3, port = urllib.splitport(host)
    h2 = '.'.join(h3.split('.')[-2:])
    online = ['everydo.cn', 'easydo.cn', 'everydo.com']
    host = 'oc-api.' + h2 if h2 in online else host + '/oc_api'
    return scheme + '://' + host



# encoding: utf-8
import os
import sys
from edo_client import get_client
from package import EdoPackage
from env import EdoEnvironment
from cmdparser import parse_args

reload(sys)
sys.setdefaultencoding('utf-8')

SYNC_COMMAND = ['clone', 'push', 'pull']

def main():

    # 命令解析
    cmd, args = parse_args()

    #  获取配置对象
    edo_env = EdoEnvironment()

    #  执行非同步命令
    if cmd not in SYNC_COMMAND:
        if cmd == 'server':
            edo_env.server(args.section)
        elif cmd == 'config':
            edo_env.config()
        return

    # 计算local_root
    pkg_name = args.pkg_name if cmd == 'clone' else ''
    path_filter = args.path_filter if cmd != 'clone' else ''
    local_root = os.path.abspath(pkg_name) if pkg_name else find_package_root(args.path_filter)
    if pkg_name and os.path.exists(local_root):
        print u'软件包已存在，请使用pull命令'
        return
    elif not local_root:
        print u'无法找到软件包根目录'
        return

    # 相对路径转换: 计算path_filter相对于local_root的子路径
    abspath = os.path.abspath(args.path_filter) if path_filter else local_root
    path_filter = '' if abspath == local_root else os.path.relpath(abspath, local_root)

    #  读取配置
    oc_api, account, instance, username, password, client_id, \
        client_secret = edo_env.load_config(args.section)

    #  确认同步操作
    print u'当前服务器配置： %s, 账号 %s, 站点 %s' % (oc_api, account, instance)
    if cmd in ['push', 'pull'] and not confirm_sync(cmd):
        print u'同步被取消'
        return

    # 生成服务端连接
    wo_client = get_client('workonline', oc_api, account, instance, username,
                           password, client_id=client_id, client_secret=client_secret)

    # 初始化包管理器
    edo_pkg = EdoPackage(wo_client.package, local_root)

    # 执行同步命令
    if cmd == 'pull':
        edo_pkg.pull(path_filter)
    elif cmd == 'push':
        edo_pkg.push(path_filter)
    elif cmd == 'clone':
        edo_pkg.clone()

def confirm_sync(cmd):
    print u'%s 命令会删除不存在的内容，您确定要继续么？' % cmd
    sys.stdout.write(u'您的选择[y/n](y:继续 | n:取消):')
    return raw_input().lower() == 'y'

def find_package_root(path):
    # 优先在目标目录下查找
    base_path = os.path.abspath(path)
    config_path = os.path.join(base_path, 'config.yaml')
    # 不存在时往上级查找
    while not os.path.exists(config_path):
        if base_path != os.path.dirname(base_path):
            base_path = os.path.dirname(base_path)
            config_path = os.path.join(base_path, 'config.yaml')
        else:
            # 到达根时退出
            return False
    return base_path

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import sys, yaml, pyaml


def main(argv=None):
	import argparse
	parser = argparse.ArgumentParser(
		description='Process and dump prettified YAML to stdout.')
	parser.add_argument('path', nargs='?', metavar='path',
		help='Path to YAML to read (default: use stdin).')
	opts = parser.parse_args(argv or sys.argv[1:])

	src = open(opts.path) if opts.path else sys.stdin
	try: data = yaml.load(src)
	finally: src.close()

	pyaml.dump(data, sys.stdout)


if __name__ == '__main__': sys.exit(main())



# encoding: utf-8

import os
import sys

VERSION = '0.1.15'

EDO_CONFIG_PATH = os.path.normpath(os.path.expanduser(r'~/.edopkgrc'))
SECTION = 'dev'
OC_API = r'https://oc-api.everydo.cn'
ACCOUNT='zopen'
INSTANCE = 'default'
USERNAME='admin'
CLIENT_ID = 'test'
CLIENT_SECRET = '022127e182a934dea7d69s10697s8ac2'
HELP_INFO = u'''
使用edopkg，可以将文件系统中的软件包和易度系统中的软件包同步。
服务器配置文件位于 ''%(PATH)s''
''' % {'PATH':EDO_CONFIG_PATH}
SERVER_HELP = u'查看和设置服务器配置'
CONFIG_HELP = u'添加和修改服务器配置'
CLONE_HELP = u'复制一个软件包'
PULL_HELP = u'下载一个软件包'
PUSH_HELP = u'上传一个软件包'



# encoding: utf-8
import sys
import argparse
from config import HELP_INFO, SERVER_HELP, CONFIG_HELP, CLONE_HELP, PULL_HELP, \
    PUSH_HELP, VERSION

class EdoArgParser(argparse.ArgumentParser):
    def error(self, message):
        print >> sys.stderr, u'错误的命令，以下是帮助信息'
        self.print_help()
        sys.exit(2)

def init_parse():
    parser = EdoArgParser(argument_default=argparse.SUPPRESS,
                          description=HELP_INFO,
                          epilog='edopkg %s' % (VERSION),
                          version=VERSION)
    p = parser.add_subparsers(title=u'命令')

    edo_server = p.add_parser('server', help=SERVER_HELP)
    edo_server.add_argument('section', nargs='?', action='store',help=u'配置名称')

    edo_config = p.add_parser('config', help=CONFIG_HELP)

    edo_clone = p.add_parser('clone', help=CLONE_HELP)
    edo_clone.add_argument('pkg_name', action='store', help=u'软件包名')
    edo_clone.add_argument('-s', '--section', nargs='?', action='store', help=u'配置名称')

    edo_pull = p.add_parser('pull', help=PULL_HELP)
    edo_pull.add_argument('path_filter', nargs='?', action='store', default='', help=u'要同步的文件夹/文件')
    edo_pull.add_argument('-s', '--section', nargs='?', action='store', help=u'配置名称')

    edo_push = p.add_parser('push', help=PUSH_HELP)
    edo_push.add_argument('path_filter', nargs='?', action='store',default='', help=u'要同步的文件夹/文件')
    edo_push.add_argument('-s', '--section', nargs='?', action='store', help=u'配置名称')
    return parser

def parse_args(argv=sys.argv[1:]):
    parser = init_parse()
    if not argv:
        parser.print_help()
        return '', ''
    return sys.argv[1], parser.parse_args(argv)

if '__name__' == '__main__':
    print parse_args()



# encoding: utf-8

from edo_client import OcClient, WoClient

def get_wo_client(oc_api, client_id, client_secret, account, instance, username, password):
    # 连接应用服务器
    print u' 正在连接到服务器... 地址：%s '% oc_api
    oc_client= OcClient(oc_api, client_id, client_secret,
            account = account)

    #  获取认证
    print u' 正在获取服务器认证... 账号：%s ' % account
    oc_client.auth_with_password(username=username,
            password=password, account=account)

    #  获取服务
    print u' 正在获取云办公服务... 站点：%s ' % instance
    wo_instance = oc_client.account.get_instance(account=account,
            application='workonline', instance=instance)
    print u'wo_api地址为: ', wo_instance['api_url']

    #  获取连接
    wo_client = WoClient(wo_instance['api_url'], client_id,
            client_secret, account=account, instance=instance)

    #  获取认证
    wo_client.auth_with_token(oc_client.access_token)
    print u' 成功获取连接： %(oc_api)s, 账号 %(account)s, 站点 %(instance)s ' % {
        'oc_api':oc_api,
        'account':account,
        'instance':instance
    }
    return wo_client

def complie_script(script):
   # 组装为函数
    lines = []
    for line in script.splitlines():
        lines.append('   ' + line)
    code = """def complie_script(s):
%s"""   % ('\n'.join(lines))
    compile(code, '', 'exec')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import itertools as it, operator as op, functools as ft
from collections import defaultdict, OrderedDict
import os, sys, io, yaml


class PrettyYAMLDumper(yaml.dumper.SafeDumper):

    def __init__(self, *args, **kws):
        self.pyaml_force_embed = kws.pop('force_embed', False)
        return super(PrettyYAMLDumper, self).__init__(*args, **kws)

    def represent_odict(dumper, data):
        value = list()
        node = yaml.nodes.MappingNode(
            'tag:yaml.org,2002:map', value, flow_style=None )
        if dumper.alias_key is not None:
            dumper.represented_objects[dumper.alias_key] = node
        for item_key, item_value in data.viewitems():
            node_key = dumper.represent_data(item_key)
            node_value = dumper.represent_data(item_value)
            value.append((node_key, node_value))
        node.flow_style = False
        return node

    def serialize_node(self, node, parent, index):
        if self.pyaml_force_embed: self.serialized_nodes.clear()
        return super(PrettyYAMLDumper, self).serialize_node(node, parent, index)

    @staticmethod
    def pyaml_transliterate(string):
        from unidecode import unidecode
        string_new = ''
        for ch in unidecode(string):
            if '0' <= ch <= '9' or 'A' <= ch <= 'Z' or 'a' <= ch <= 'z' or ch in '-_': string_new += ch
            else: string_new += '_'
        return string_new.lower()

    def anchor_node(self, node, hint=list()):
        if node in self.anchors:
            if self.anchors[node] is None and not self.pyaml_force_embed:
                self.anchors[node] = self.generate_anchor(node)\
                    if not hint else '{}'.format(
                        self.pyaml_transliterate(
                            '_-_'.join(map(op.attrgetter('value'), hint)) ) )
        else:
            self.anchors[node] = None
            if isinstance(node, yaml.nodes.SequenceNode):
                for item in node.value:
                    self.anchor_node(item)
            elif isinstance(node, yaml.nodes.MappingNode):
                for key, value in node.value:
                    self.anchor_node(key)
                    self.anchor_node(value, hint=hint+[key])

PrettyYAMLDumper.add_representer(defaultdict, PrettyYAMLDumper.represent_dict)
PrettyYAMLDumper.add_representer(set, PrettyYAMLDumper.represent_list)
PrettyYAMLDumper.add_representer(OrderedDict, PrettyYAMLDumper.represent_odict)


class UnsafePrettyYAMLDumper(PrettyYAMLDumper):

    def choose_scalar_style(self):
        if self.event.style != 'plain':
            return super(UnsafePrettyYAMLDumper, self).choose_scalar_style()
        else:
            return "'"

    def expect_block_sequence(self):
        self.increase_indent(flow=False, indentless=False)
        self.state = self.expect_first_block_sequence_item

    def expect_block_sequence_item(self, first=False):
        if not first and isinstance(self.event, yaml.events.SequenceEndEvent):
            self.indent = self.indents.pop()
            self.state = self.states.pop()
        else:
            self.write_indent()
            self.write_indicator('-', True, indention=True)
            self.states.append(self.expect_block_sequence_item)
            self.expect_node(sequence=True)

    def represent_stringish(dumper, data):
        # Will crash on bytestrings with weird chars in them,
        #  because we can't tell if it's supposed to be e.g. utf-8 readable string
        #  or an arbitrary binary buffer, and former one *must* be pretty-printed
        # PyYAML's Representer.represent_str does the guesswork and !!binary or !!python/str
        # Explicit crash on any bytes object might be more sane, but also annoying
        # Use something like base64 to encode such buffer values instead
        # Having such binary stuff pretty much everywhere on unix (e.g. paths) kinda sucks
        data = unicode(data) # read the comment above

        # Try to use '|' style for multiline data,
        #  quoting it with 'literal' if lines are too long anyway,
        #  not sure if Emitter.analyze_scalar can also provide useful info here
        if '\n' in data:
            style = '|'
            data = '\n'.join([line.rstrip() for line in data.splitlines()])
        else:
            style = 'plain'

        return yaml.representer.ScalarNode('tag:yaml.org,2002:str', data, style=style)

for str_type in [bytes, unicode]:
    UnsafePrettyYAMLDumper.add_representer(
        str_type, UnsafePrettyYAMLDumper.represent_stringish )

UnsafePrettyYAMLDumper.add_representer(
    type(None), lambda s,o: s.represent_scalar('tag:yaml.org,2002:null', '') )


def dump_add_vspacing(buff, vspacing):
    'Post-processing to add some nice-ish spacing for deeper map/list levels.'
    if isinstance(vspacing, int):
        vspacing = ['\n']*(vspacing+1)
    buff.seek(0)
    result = list()
    for line in buff:
        level = 0
        line = line.decode('utf-8')
        result.append(line)
        if ':' in line:
            while line.startswith('  '):
                level, line = level + 1, line[2:]
            if len(vspacing) > level and len(result) != 1:
                vspace = vspacing[level]
                result.insert( -1, vspace
                    if not isinstance(vspace, int) else '\n'*vspace )
    buff.seek(0), buff.truncate()
    buff.write(''.join(result).encode('utf-8'))


def dump(data, dst=unicode, safe=False, force_embed=False, vspacing=None):
    buff = io.BytesIO()
    Dumper = PrettyYAMLDumper if safe else UnsafePrettyYAMLDumper
    Dumper = ft.partial(Dumper, force_embed=force_embed)
    yaml.dump_all([data], buff, Dumper=Dumper, default_flow_style=False, encoding='utf-8', allow_unicode=True, indent=4)

    if vspacing is not None:
        dump_add_vspacing(buff, vspacing)

    if dst is bytes:
        return buff.getvalue()
    elif dst is unicode:
        return buff.getvalue().decode('utf-8')
    else:
        dst.write(buff.getvalue())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
#
#   ZopeEdit, client for ExternalEditor
#
##############################################################################

import os
from setuptools import setup, find_packages
from edopkg import config

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
setup(
        name='edopkg',
        version=config.VERSION,
        zip_safe=False,
        author="xiehaiyang",
        author_email="service@everydo.com",
        url="http://everydo.com/",
        description=u"Easydo Package Syncer",
        long_description=README,
        packages = find_packages(),
        entry_points = {
            'console_scripts': ['edopkg = edopkg.main:main']
        },
        install_requires=[
            'edo_client',
            'PyYAML'
        ],
)



# encoding: utf-8
import os
import sys
import shutil
import yaml
import traceback
import logging
from edopkg import pyaml
from utils import complie_script

# 优先检测代码并上传， 最后上传form
OBJ_CATS = ['script', 'workflow', 'mdset', 'rule', 'template',
        'stage','skin', 'resource', 'form']

class EdoPackage:

    def __init__(self, remote_pkg, local_root):
        self.remote_pkg = remote_pkg
        self.local_root = local_root
        self.pkg_name = os.path.basename(local_root)
        logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                            datefmt='%Y-%m-%d %H:%M:%S',
                            stream=sys.stdout
                            )
        logger=logging.getLogger('sync')
        logger.setLevel(logging.INFO)
        self.logger = logger

    def clone(self):
        os.makedirs(self.local_root)
        try:
            self.pull()
        except Exception as e:
            shutil.rmtree(self.local_root)
            logging.error(u'命令:clone执行失败，软件包：%s,  错误信息：%s' %(self.pkg_name, e))
            raise

    def pull(self, path_filter=''):
        if not path_filter or path_filter == 'config.yaml':
            self.logger.info(u'pushing config.yaml...')
            pkg_info = self.remote_pkg.get(self.pkg_name, detail=False)
            self.write_yaml('config.yaml', pkg_info)

        # 写入软件包对象
        for obj_cat in OBJ_CATS:
            obj_folder = obj_cat + 's'

            # 过滤
            if path_filter and not path_filter.startswith(obj_folder):
                continue

            # 计算文件的过滤条件
            filename_filter = path_filter[len(obj_folder) + 1:]

            #  删除同步：清空本地文件夹
            if not filename_filter:
                abs_folder_path = os.path.join(self.local_root, obj_folder)
                path_list = os.listdir(abs_folder_path) if os.path.exists(abs_folder_path) else []
                self.logger.info(u'清空%s文件夹...' % obj_folder)
                for ph in path_list:
                    abspath = os.path.join(abs_folder_path, ph)
                    os.remove(abspath) if os.path.isfile(abspath) else shutil.rmtree(abspath)

            # 逐个对象同步
            for obj_name in getattr(self.remote_pkg, 'list_' + obj_folder)(self.pkg_name):
                self.pull_obj(obj_cat, obj_name, obj_folder, filename_filter)

        self.logger.info(u'下载成功')

    def push(self, path_filter=''):

        if not path_filter or path_filter == 'config.yaml':
            self.logger.info(u'pushing config.yaml...')
            pkg_info = self.read_yaml('config.yaml')
            pkg_list = self.remote_pkg.list()
            if self.pkg_name not in pkg_list:
                status = self.remote_pkg.new(self.pkg_name, pkg_info)
            status = self.remote_pkg.set(self.pkg_name, pkg_info)

        # 读取软件包对象
        for obj_cat in OBJ_CATS:
            obj_folder = obj_cat + 's'
            # 过滤
            if path_filter and not path_filter.startswith(obj_folder):
                continue
            filename_filter = path_filter[len(obj_folder) + 1:]


            # 删除同步：删除远程的对象
            if not filename_filter:
                self.logger.info(u'清空远程%s...' % obj_folder)
                for obj_name in getattr(self.remote_pkg, 'list_' + obj_folder)(self.pkg_name):
                    if obj_cat == 'resource':
                        self.remote_pkg.remove_resource(self.pkg_name, obj_name)
                    else:
                        getattr(self.remote_pkg, 'remove_' + obj_cat)(obj_name)

            # 判断是否存在文件夹
            cat_path = os.path.join(self.local_root, obj_folder)
            if not os.path.exists(cat_path):
                continue

            # 获取同步列表
            obj_list = os.listdir(cat_path)

            # 根据文件类型进行过滤
            obj_list = filter(lambda f:not f.startswith(('.', '~')), obj_list)
            obj_list = filter(lambda f:not f.endswith(('.bak', '~')), obj_list)
            if obj_cat == 'template':
                obj_list == filter(lambda f:f.endswith('.pt'), obj_list)
            elif obj_cat != 'resource':
                obj_list == filter(lambda f:f.endswith('.yaml'), obj_list)

            # 对于表单进行特殊处理
            if obj_cat == 'form':
                self.logger.info(u'分析表单类型')
                first_list, after_list = [], []
                for obj_filename in obj_list:
                    obj_data = self.read_yaml(os.path.join(self.local_root, cat_path, obj_filename))
                    # 优先上传类型为DataContainer的表单
                    if ''.join(obj_data['object_types']) == 'DataContainer':
                        first_list.append(obj_filename)
                    else:
                        after_list.append(obj_filename)
                # 合并列表
                obj_list = first_list + after_list

            # 逐个上传
            for obj_filename in obj_list:
                # 过滤
                if filename_filter and not obj_filename.startswith(filename_filter):
                    continue

                #  同步
                if os.path.isdir(os.path.join(self.local_root, obj_folder, obj_filename)):
                    self.push_folder(os.path.join(obj_folder, obj_filename))
                else:
                    self.push_obj(obj_cat, obj_folder, obj_filename)

        self.logger.info(u'上传成功')

    def push_folder(self, obj_folder, obj_cat='resource'):
        for obj_filename in os.listdir(os.path.join(self.local_root, obj_folder)):
            if os.path.isdir(os.path.join(self.local_root, obj_folder, obj_filename)):
                self.push_folder(os.path.join(obj_folder, obj_filename))
            else:
                self.push_obj(obj_cat, obj_folder, obj_filename)

    def pull_obj(self, obj_cat, obj_name, obj_folder, filename_filter):
        # 读取文件名
        if obj_cat == 'resource':
            obj_filename = os.path.normpath(obj_name)
        elif obj_cat == 'template':
           obj_filename = obj_name.split(':')[1] + '.pt'
        else:
           obj_filename = obj_name.split(':')[1] + '.yaml'

        # 过滤
        if filename_filter and not obj_filename.startswith(filename_filter):
            return

        # 更新提示
        self.logger.info('pulling %s %s' % (obj_cat, obj_name))

        # 下载数据
        if obj_cat == 'resource':
            obj_data = self.remote_pkg.get_resource(self.pkg_name, res_path = obj_name)
        else:
            obj_data = getattr(self.remote_pkg, 'get_' + obj_cat)(obj_name)

        obj_path = os.path.join(obj_folder, obj_filename)

        # 判断文件夹是否存在
        dirname = os.path.dirname(os.path.join(self.local_root, obj_path))
        if not os.path.exists( dirname):
            os.makedirs(dirname)

        # 写入数据
        if obj_cat == 'template':
            self.write_file(obj_path, obj_data['template'])
        elif obj_cat == 'resource':
            self.write_response(obj_path, obj_data)
        else:
            self.write_yaml(obj_path, obj_data)

    def push_obj(self, obj_cat, obj_folder, obj_filename):
        obj_name = obj_filename.split('.')[0]
        # 读取数据
        obj_path = os.path.join(obj_folder, obj_filename)
        if obj_cat == 'template':
            obj_data = self.read_template(obj_name, obj_path)
        elif obj_cat == 'resource':
            obj_stream = self.read_stream(obj_path)
        else:
            obj_data = self.read_yaml(obj_path)

        # 代码检测
        if obj_cat == 'script':
            script = obj_data.get('script', '')
            try:
                complie_script(script)
            except:
                self.logger.warning(u'脚本代码 %s 存在错误，以下是错误信息：' % obj_filename)
                traceback.print_exc(0)
                self.logger.error(u'上传中止')
                sys.exit(2)

        # 上传数据
        self.logger.info('pushing %s: %s' % (obj_filename, obj_folder))
        if obj_cat == 'template':
            status = getattr(self.remote_pkg, 'register_' + obj_cat)(
                self.pkg_name, obj_data, overwrite = True)
        elif obj_cat == 'resource':
            remote_path = '/'.join([obj_folder[len('resources')+1:], obj_filename])
            status = self.remote_pkg.add_resource(
                self.pkg_name, remote_path, obj_stream, overwrite=True)
        else:
            status = getattr(self.remote_pkg, 'register_' + obj_cat)(
                self.pkg_name, obj_data, overwrite = True)

    def write_yaml(self, path, data):
        open(os.path.join(self.local_root, path), 'wb').write(pyaml.dump(data))

    def write_file(self, path, data='', response=''):
        open(os.path.join(self.local_root, path), 'wb').write(data)

    def write_response(self, path, response):
        with open(os.path.join(self.local_root, path), 'wb') as f:
            for block in response.iter_content(chunk_size=1024):
                if not block:
                    break
                f.write(block)
                f.flush()

    def read_yaml(self, path):
        return yaml.load(open(os.path.join(self.local_root, path)).read())

    def read_template(self, obj_name, path):
        data = open(os.path.join(self.local_root, path)).read()
        return {'name':obj_name, 'title':obj_name, 'template':data}

    def read_stream(self, path):
        return open(os.path.join(self.local_root, path))

